@@ -8,22 +8,31 @@
 ##   Created: Oct 20, 2009
 ##
 
-## modelled on print methods in the cluster package
+## print method for Ckmeans.1d.dp object
 print.Ckmeans.1d.dp <- function(x, ...)
 {
-  if(length(x$size) > 1) {
-    cat("Ckmeans.1d.dp returns ", length(x$size), " clusters, each containing ",
-        paste(x$size, collapse=", "), " elements, respectively.\n", sep="")
-  } else {
-    cat("Ckmeans.1d.dp returns one cluster containing all elements.\n")
-  }
+  with(x, {
+    if(length(size) > 1) {
+      cat("Ckmeans.1d.dp returns", length(size), "optimal clusters of sizes",
+          paste(size, collapse=", "), "\n")
+      cat("  minimum total within-cluster sum of squares:", tot.withinss, "\n")
+      cat("  maximum between-cluster sum of squares:", betweenss, "\n")
+      cat("  total sum of squares of input vector:", totss, "\n")
+      if(totss > 0) {
+        cat("  maximum (between-SS / total-SS):",
+            round(1000 * betweenss / totss)/10, "%\n")
+      }
+    } else {
+      cat("Ckmeans.1d.dp returns one cluster containing all elements.\n")
+    }
+    cat("\nCluster means:\n")
+    print(centers, ...)
+    cat("\nCluster index associated with each element:\n")
+    print(cluster, ...)
+    cat("\nWithin-cluster sum of squares:\n")
+    print(withinss, ...)
+  })
 
-  cat("\nCluster means:\n")
-  print(x$centers, ...)
-  cat("\nCluster id of each element:\n")
-  print(x$cluster, ...)
-  cat("\nWithin-cluster sum of squares:\n")
-  print(x$withinss, ...)
   cat("\nAvailable components:\n")
   print(names(x))
   invisible(x)
@@ -46,20 +55,20 @@ Ckmeans.1d.dp <- function( x, k=c(1,9) )
     k.max <- k
   }
 
-	##Check to see if k is less than 0.
-	##If k is less than 0, stop and exit.
+  ##Check to see if k is less than 0.
+  ##If k is less than 0, stop and exit.
   if( k.max <= 0 ) {
-		stop ("Can NOT classify vector into 0 or less cluster\n")
+    stop ("Can NOT classify vector into 0 or less cluster\n")
   }
 
   n.unique <- length(unique(x))
-	##Check to see if cluster level bigger than the unique number of the input vector
-	##If k is bigger than the unique number of the input vector,
-	##force k set to the number of unique number of input.
+  ##Check to see if cluster level bigger than the unique number of the input vector
+  ##If k is bigger than the unique number of the input vector,
+  ##force k set to the number of unique number of input.
   if(n.unique < k.min) {
-		warning("Min number of clusters is greater than the unique number of elements in\n",
-        "the input vector, both k.min and k.max are set to the number of\n",
-        "unique number of input values.\n")
+    warning("Min number of clusters is greater than the unique number of elements in\n",
+            "the input vector, both k.min and k.max are set to the number of\n",
+            "unique number of input values.\n")
     k.min <- n.unique
     k.max <- n.unique
   } else if (n.unique >= k.min && n.unique < k.max) {
@@ -69,13 +78,13 @@ Ckmeans.1d.dp <- function( x, k=c(1,9) )
     k.max <- n.unique
   }
 
-	##Form data which will be passed to external C++ function.
+  ##Form data which will be passed to external C++ function.
   clusters <- vector("integer", length(x))
   center <- vector("double", k.max)
   withinss <- vector("double", k.max)
   size <- vector("integer", k.max)
 
-	#Call external C++ function
+  #Call external C++ function
   result <- .C("Ckmeans_1d_dp", PACKAGE="Ckmeans.1d.dp",
                data=as.double(x), length=as.integer(length(x)),
                Kmin=as.integer(k.min), Kmax=as.integer(k.max),
@@ -92,9 +101,13 @@ Ckmeans.1d.dp <- function( x, k=c(1,9) )
     }
   }
 
+  totss <- sum(scale(x, scale=FALSE)^2)
+  tot.withinss <- sum(result$withinss[1:k.opt])
+  betweenss <- totss - tot.withinss
   r <- structure(list(cluster = result$cluster, centers = result$centers[1:k.opt],
-                      withinss = result$withinss[1:k.opt], size = result$size[1:k.opt]),
+                      withinss = result$withinss[1:k.opt], size = result$size[1:k.opt],
+                      totss = totss, tot.withinss = tot.withinss, betweenss = betweenss),
                  class = "Ckmeans.1d.dp")
 
-	return( r )
+  return( r )
 } ##end of Ckmeans.1d.dp()
